/* ========================
 * JSynoptic : a free Synoptic editor
 * ========================
 *
 * Project Info:  http://jsynoptic.sourceforge.net/index.html
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * (C) Copyright 2001-2003, by :
 *     Corporate: 
 *         Astrium SAS 
 *         EADS CRC
 *     Individual: 
 * 		   Christophe Pecquerie
 *
 * $Id: TspDataPlugin.java,v 1.2 2005/09/14 16:18:05 erk Exp $
 * 
 * Changes ------- 06-Jan-2004 : Creation Date (NB);
 *  
 */
package tsp.consumer.plugin.jsynoptic;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import jsynoptic.base.Plugin;
import jsynoptic.ui.JSynoptic;
import simtools.data.DataInfo;
import simtools.data.DataSource;
import simtools.data.DataSourceCollection;
import simtools.data.DataSourcePool;
import simtools.data.DataSourceProvider;
import simtools.data.DuplicateIdException;
import simtools.ui.MenuResourceBundle;
import simtools.ui.ResourceFinder;
import tsp.consumer.plugin.jsynoptic.impl.TspHandler;
import tsp.consumer.plugin.jsynoptic.impl.TspSampleSymbolInfo;
import tsp.consumer.plugin.jsynoptic.impl.TspHandler.TspProviderNotFoundException;
import tsp.consumer.plugin.jsynoptic.ui.TspDialogOpenProvider;
import tsp.core.common.url.TspMalformedURLException;
import tsp.core.common.url.TspURL;
import tsp.core.common.url.TspURLException;
import tsp.core.common.url.TspURLFactory;

public class TspDataPlugin extends Plugin implements DataSourceProvider, ActionListener{
	
    static MenuResourceBundle resources = ResourceFinder.getMenu(JSynoptic.class);
    static MenuResourceBundle tspResources = ResourceFinder.getMenu(TspDataPlugin.class);

    protected JMenuItem jmi;
	
	protected static MenuResourceBundle.FileFilter urlFilter;
	static javax.swing.filechooser.FileFilter[] filters;
	static {
		urlFilter = tspResources.getFileFilter("tspFileFilter");
		filters=new javax.swing.filechooser.FileFilter[] {
				urlFilter,
		};
	}
	
	public TspDataPlugin() {
		DataSourcePool.global.addProvider(this);
	}

	public javax.swing.filechooser.FileFilter[] getFileFilters(int action) {
		if (action!=OPEN) return null;
		return filters;
	}
	
	public boolean processFile(File f, int action) {
		TspURL url=null;
		double samplingFrequency = -1.;
		int samplingPhase = 0;
		
		// first try to read the file content in case it contains the URL
		try {
			RandomAccessFile raf = new RandomAccessFile(f, "r");
			String line=raf.readLine();
			if(line!=null && !line.trim().isEmpty()){
				url=TspURLFactory.createWithDefaultSupport(line.trim()); 
			}
			// if there is a second line then try to read frequency, phase
			line=raf.readLine();
			if(line!=null && !line.isEmpty()){
				int comma=line.indexOf(',');
				if(comma>0){
					try{
						samplingPhase=Integer.parseInt(line.substring(comma+1));
					}
					catch(NumberFormatException nfe){
					}
					line=line.substring(0, comma);
				}
				try{
					samplingFrequency=Double.parseDouble(line);
				}
				catch(NumberFormatException nfe){
				}				
			}
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		} catch (TspMalformedURLException e) {
		}
		// else try to get the URL from the filename using @ as a separator
		if(url==null){
			String name=f.getName();
			if(name.endsWith(".tsp")){
				name=name.substring(0, name.lastIndexOf(".tsp"));
			}
			String[] res=name.split("@");
			if(res!=null && res.length>1){
				try {
					url=TspURLFactory.createEmpty();
					url.setProtocol(res[0]);
					url.setHost(res[1]);
					if(res.length>2){
						url.setServerName(res[2]);
					}
					if(res.length>3){
						try{
							Integer i=Integer.valueOf(res[3]);
							url.setServerNumber(i);
						}
						catch(NumberFormatException nfe){
						}
					}
				} catch (TspMalformedURLException e) {
				}			
				// if there are other parameters try to read frequency, phase
				if(res.length>4){
					try{
						samplingFrequency=Double.parseDouble(res[4]);
					}
					catch(NumberFormatException nfe){
					}
				}
				if(res.length>5){
					try{
						samplingPhase=Integer.parseInt(res[5]);
					}
					catch(NumberFormatException nfe){
					}
				}
			}
		}
		
		if(url!=null){
			try{
				TspHandler handler=new TspHandler(url);
				if(samplingPhase!=0){
					handler.setSamplingPhase(samplingPhase);
				}
				int tempPeriod = (int) Math.round(handler.getProviderBaseFrequency()/samplingFrequency);
				if(tempPeriod>=1 && tempPeriod<=handler.getProviderMaxPeriod()){
					handler.setPeriod_(tempPeriod);
				}
				TspDataSourceCollection tdsc = new TspDataSourceCollection(handler);
				DataSourcePool.global.addDataSourceCollection(tdsc);
				return true;
			}
			catch(TspURLException e1){
			}
			catch(TspProviderNotFoundException e2){
			}
		}
		return false;
	}

	/** 
	 * Adds a menu entry in the file menu
	 */
	public void setMenu(JMenuBar mb) {
		Component c[] = mb.getComponents();
		for (int i=0; i<c.length; ++i) {
			JMenu jm = (JMenu)c[i];
			//Search for the file menu
			if (jm.getText().equals(resources.getString("fileMenu"))) {
				Component cm[] = jm.getMenuComponents();
				for (int j=0; j<cm.length; ++j) {
					if(cm[j] instanceof JMenuItem){
						JMenuItem jsm = (JMenuItem)cm[j];
						//Search for the menu item "open"
						if (jsm.getText().equals(resources.getString("openMenu"))) {
							jmi = new JMenuItem("Open TSP Provider...");
							jm.add(jmi,2);
							jmi.addActionListener(this);
						}
					}
				}
			}
		}
	}
	
	//Returns the plugin information
	public String about() {
		return "TSP sources, a TSP consumer plugin by Christophe Pecquerie";
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		//Open dialog to connect to a new Tsp provider
		TspDialogOpenProvider windowOpenProvider = new TspDialogOpenProvider();
		TspHandler tspHandler;
		tspHandler = windowOpenProvider.getTspHandler();
		if (tspHandler != null)
		{
			//Create data sources collection
			TspDataSourceCollection tdsc = new TspDataSourceCollection(tspHandler);
			DataSourcePool.global.addDataSourceCollection(tdsc);
		}
	}
	
	/* (non-Javadoc)
	 * @see simtools.data.DataSourceProvider#getOptionalInformation(simtools.data.DataSource, simtools.data.DataSourceCollection)
	 */
	public Object getOptionalInformation(DataSource ds, DataSourceCollection dsc) {
		if ((dsc==null) || (!(dsc instanceof TspDataSourceCollection)))
			return null;

		TspDataSourceCollection tdsc = (TspDataSourceCollection) dsc;

		// As we have only random sources, we can easily re-create those.
		// Just storing the number of sources we have is enough
		return tdsc.tspHandler_;
	}

	/* (non-Javadoc)
	 * @see simtools.data.DataSourceProvider#provide(java.lang.String, java.lang.String, java.lang.Object, simtools.data.DataSourcePool)
	 */
	public DataSource provide(String id, String dscId, Object optionalInformation, DataSourcePool pool) {
		// Ignore requests that are not for this object
		if ((dscId==null) || (!dscId.startsWith("TSP"))) return null;
		
		// Usually, a provider may not need the pool  
		// In this example, we suppose there is only one instance, and that it was put in the
		// pool by this plugin. So, if the pool is not the global one, error
		if (pool!=DataSourcePool.global) return null;
		
		// Get the unique instance
		TspDataSourceCollection tdsc;
		try {
			//Get the optional information provided by the deserialization
			TspHandler tspHandler = (TspHandler)optionalInformation;
			//Get the corresponding collection
			tdsc = (TspDataSourceCollection)pool.getDataSourceCollectionWithId(tspHandler.getId());
			if (tdsc == null) {				
				TspDialogOpenProvider windowOpenProvider = new TspDialogOpenProvider(tspHandler);
				tspHandler = windowOpenProvider.getTspHandler();
				
				if(tspHandler == null)	return null;
				
				tdsc = new TspDataSourceCollection(tspHandler);
				DataSourcePool.global.addDataSourceCollection(tdsc);
			}
			//Find the source in tspHandler's symbols list
			for(int i=0; i<tdsc.tspHandler_.getSymbolTab().length ; i++)
				if(tdsc.tspHandler_.getSymbolTab()[i].name.equals(id)) {
					TspSampleSymbolInfo symbol = tdsc.tspHandler_.getSymbolTab()[i];
					tdsc.addSource(new DataInfo(symbol.name,symbol.name,"Symbol from " + tdsc.tspHandler_.getUrl().getHost() + ":" + tdsc.tspHandler_.getProviderChannelId(),"NA"));
					//Request this symbol to the provider
					tdsc.tspHandler_.getSymbolTab()[i].sample = true;
					break;
				}
			// Now we're confident all sources were re-created, so return the source for this id
			return tdsc.get(id);
		} catch (DuplicateIdException e) {
			e.printStackTrace();
			return null;
		}
	}
}
